// This is the entry point to the auto-updater, and should
// be required by the main electron process.
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var _a = require('electron'), autoUpdater = _a.autoUpdater, ipcMain = _a.ipcMain, dialog = _a.dialog, app = _a.app;
var winston = require('winston');
var logFormatter = winston.format.printf(function (info) {
    var timestamp = info.timestamp, level = info.level, code = info.code, stack = info.stack, message = info.message, rest = __rest(info
    // print out http error code w/ a space if we have one
    , ["timestamp", "level", "code", "stack", "message"]);
    // print out http error code w/ a space if we have one
    code = code ? " " + code : '';
    // print the stack if we have it, message otherwise.
    message = stack || message;
    return "> " + timestamp + " " + level + code + ": " + message + " " + (Object.keys(rest).length !== 0 ? "\n" + JSON.stringify(rest, null, 2) : '');
});
winston.configure({
    level: 'info',
    // put the errors formatter in the parent for some reason, only needed there:
    format: winston.format.errors({ stack: true }),
    transports: [
        new winston.transports.File({
            filename: "updater.log",
            format: winston.format.combine(winston.format.timestamp(), logFormatter)
        })
    ]
});
var getPlatform = function () {
    switch (process.platform) {
        case 'win32':
            return process.arch === 'x64' ? 'win64' : 'win32';
        case 'darwin':
            return 'darwin';
    }
};
var runUpdater = function (mainWindow) {
    winston.log('info', "[Updater]: v" + app.getVersion() + " ================================");
    autoUpdater.on('checking-for-update', function (info) {
        winston.log('info', "autoUpdater.on('checking-for-update')", info);
        mainWindow.webContents.send('updaterEvent', {
            event: 'checking-for-update',
            data: info
        });
    });
    autoUpdater.on('update-available', function (info) {
        winston.log('info', "autoUpdater.on('update-available')", info);
        mainWindow.webContents.send('updaterEvent', {
            event: 'update-available',
            data: info
        });
    });
    autoUpdater.on('update-not-available', function (info) {
        winston.log('info', "autoUpdater.on('update-not-available')", info);
        mainWindow.webContents.send('updaterEvent', {
            event: 'update-not-available',
            data: info
        });
    });
    autoUpdater.on('update-downloaded', function (event, releaseNotes, releaseName, releaseDate, updateURL) {
        // const info = { releaseNotes, releaseName, releaseDate, updateURL }
        mainWindow.webContents.send('updaterEvent', {
            event: 'update-downloaded',
            data: null
        });
        winston.log('info', "autoUpdater.on('update-downloaded')");
    });
    autoUpdater.on('error', function (info) {
        winston.log('info', "autoUpdater.on('error') fucker: " + info);
        winston.log('error', info);
        mainWindow.webContents.send('updaterEvent', {
            event: 'error',
            data: info
        });
    });
    ipcMain.handle('autoUpdater-showDialog', function (_event, _info) {
        // const { releaseNotes, releaseName } = info
        var dialogOpts = {
            type: 'info',
            buttons: ['Restart', 'Later'],
            title: 'Application Update',
            // message: process.platform === 'win32' ? releaseNotes : releaseName,
            detail: 'A new version has been downloaded. Restart the application to apply the updates.'
        };
        dialog.showMessageBox(dialogOpts).then(function (returnValue) {
            if (returnValue.response === 0)
                autoUpdater.quitAndInstall();
        });
    });
    try {
        var platform = getPlatform();
        var server = 'http://muteme.io';
        var url = server + "/update/" + platform + "/" + app.getVersion();
        autoUpdater.setFeedURL(url);
        winston.log('info', "initiating version check at " + url);
        autoUpdater.checkForUpdates();
    }
    catch (error) {
        winston.log('info', "checkForUpdates Error " + error);
        winston.log('error', error);
        mainWindow.webContents.send('updaterEvent', {
            event: 'checking-for-update',
            data: error
        });
    }
};
exports.runUpdater = runUpdater;
//# sourceMappingURL=Updater.js.map