'use strict'
const path = require('path')

const execa = require('execa')
const electronUtil = require('electron-util/node')

const binaries = {
  mute: path.join(electronUtil.fixPathForAsarUnpack(__dirname), 'scripts/mute.sh'),
  unmute: path.join(electronUtil.fixPathForAsarUnpack(__dirname), 'scripts/unmute.sh'),
  isMuted: path.join(electronUtil.fixPathForAsarUnpack(__dirname), 'scripts/is-muted.sh'),
  toggleMute: path.join(electronUtil.fixPathForAsarUnpack(__dirname), 'scripts/toggle-mute.sh')
}

const generateExport = (name, getArgs, callback) => {
  module.exports[name] = async (...inputs) => {
    const result = await execa(binaries[name], getArgs(...inputs), {
      env: {
        XDG_RUNTIME_DIR: '/run/user/1000'
      }
    })
    return callback(result)
  }

  module.exports[name].sync = (...inputs) => {
    const result = execa.sync(binaries[name], getArgs(...inputs))
    return callback(result)
  }
}

const throwIfStderr = result => {
  const { stderr } = result
  if (stderr) {
    throw new Error(stderr)
  }
}

const parseMuteStatus = result => {
  const { stdout } = result
  throwIfStderr(result)
  const out = stdout.split('\n')
  return !out.some(e => e === 'no')
}

const parseMuteUnmute = result => {
  const { stderr } = result
  // throwIfStderr(result)
  return !stderr
}

generateExport('mute', () => [], parseMuteUnmute)

generateExport('unmute', () => [], parseMuteUnmute)

generateExport('isMuted', () => [], parseMuteStatus)

generateExport('toggleMute', () => [], parseMuteUnmute)

// async function hey() {
//   // let isMuted = await module.exports.isMuted()
//   // console.log(`isMuted? ${isMuted}.`)
//   // await module.exports.mute()
//   // isMuted = await module.exports.isMuted()
//   // console.log(`isMuted? ${isMuted}. should be true`)
//   // await module.exports.unmute()
//   // isMuted = await module.exports.isMuted()
//   // console.log(`isMuted? ${isMuted}. should be false`)
// }

// hey()
