'use strict'
const path = require('path')

const execa = require('execa')
const electronUtil = require('electron-util/node')

const binary = path.join(electronUtil.fixPathForAsarUnpack(__dirname), 'audio-devices')

const generateExport = (name, getArgs, callback) => {
  module.exports[name] = async (...inputs) => {
    const result = await execa(binary, getArgs(...inputs))
    return callback(result)
  }

  module.exports[name].sync = (...inputs) => {
    const result = execa.sync(binary, getArgs(...inputs))
    return callback(result)
  }
}

const throwIfStderr = ({ stderr }) => {
  if (stderr) {
    throw new Error(stderr)
  }
}

const parseStdout = ({ stdout, stderr }) => {
  throwIfStderr({ stderr })
  return JSON.parse(stdout)
}

generateExport('getAllDevices', () => ['list', '--json'], parseStdout)

generateExport('getInputDevices', () => ['list', '--input', '--json'], parseStdout)

generateExport('getOutputDevices', () => ['list', '--output', '--json'], parseStdout)

generateExport('getDevice', deviceId => ['get', '--json', deviceId], parseStdout)

generateExport('getDefaultOutputDevice', () => ['output', 'get', '--json'], parseStdout)

generateExport('getDefaultInputDevice', () => ['input', 'get', '--json'], parseStdout)

generateExport('getDefaultSystemDevice', () => ['system', 'get', '--json'], parseStdout)

generateExport('setDefaultOutputDevice', deviceId => ['output', 'set', deviceId], throwIfStderr)

generateExport('setDefaultInputDevice', deviceId => ['input', 'set', deviceId], throwIfStderr)

generateExport('setDefaultSystemDevice', deviceId => ['system', 'set', deviceId], throwIfStderr)

generateExport(
  'getDeviceVolume',
  deviceId => ['volume', 'get', deviceId],
  ({ stdout, stderr }) => (stderr ? undefined : stdout)
)

generateExport(
  'setDeviceVolume',
  (deviceId, volume) => ['volume', 'set', deviceId, volume],
  throwIfStderr
)

generateExport(
  'getDeviceMute',
  deviceId => ['mute', 'get', deviceId],
  ({ stdout, stderr }) => (stderr ? undefined : stdout == 'true')
)

generateExport('toggleDeviceMute', deviceId => ['mute', 'toggle', deviceId], throwIfStderr)

generateExport(
  'createAggregateDevice',
  (name, mainDeviceId, otherDeviceIds, { multiOutput } = {}) =>
    [
      'aggregate',
      'create',
      '--json',
      multiOutput && '--multi-output',
      name,
      mainDeviceId,
      ...otherDeviceIds
    ].filter(Boolean),
  parseStdout
)

generateExport(
  'destroyAggregateDevice',
  deviceId => ['aggregate', 'destroy', deviceId],
  throwIfStderr
)

// async function hey() {
//   const { id: deviceId } = await module.exports.getDefaultInputDevice()
//   console.log(`default input device id: ${deviceId}`)
//   let isMuted = await module.exports.getDeviceMute(deviceId)
//   console.log(`isMuted? ${isMuted}`)
//   await module.exports.toggleDeviceMute(deviceId)
//   isMuted = await module.exports.getDeviceMute(deviceId)
//   console.log(`isMuted? ${isMuted}`)
// }

// hey()
