export const getZoomMute: {
  /**
  Get Zoom's mute status.
  @returns A promise that resolves Zoom's mute status.
  @example
  ```
  const zoomMute = await getZoomMute();
  ```
  */
  (): Promise<'muted' | 'unmuted' | boolean>
}
export const muteZoom: {
  /**
  Mutes the active Zoom window if its unmuted.
  @returns A promise that resolves with true if the active Zoom window was muted, false otherwise.
  @example
  ```
  const success = await muteZoom();
  ```
  */
  (): Promise<boolean>
}
export const unmuteZoom: {
  /**
  Unmutes the active Zoom window if its muted.
  @returns A promise that resolves with true if the active Zoom window was unmuted, false otherwise.
  @example
  ```
  const success = await unmuteZoom();
  ```
  */
  (): Promise<boolean>
}

export type ZoomControl = {
  getZoomMute: typeof getZoomMute
  muteZoom: typeof muteZoom
  unmuteZoom: typeof unmuteZoom
}
