const os = require('os')

if (os.platform() === 'win32') {
  const runWorker = require('./build/Release/zoom-control')
  const { promisify } = require('util')

  const getZoomStatusAsync = promisify(runWorker.getZoomStatus)
  const toggleZoomMuteAsync = promisify(runWorker.toggleZoomMute)

  const getZoomMute = async () => {
    const result = await getZoomStatusAsync()
    const status = result
      .split(',')
      .reduce((acc, cur) => ({ ...acc, [`${cur.split(':')[0]}`]: cur.split(':')[1] }), {})
    const zoomMute =
      (status?.zoomStatus === 'call' || status?.zoomStatus === 'call-sharing') && status?.zoomMute
    console.log(`[getZoomMute()]: ${JSON.stringify(status)}. returning isMuted=${zoomMute}`)
    return zoomMute
  }
  const muteZoom = async () => {
    const zoomMute = await getZoomMute()
    if (zoomMute === 'unmuted') {
      console.log(`[muteZoom()]: muting Zoom by toggling the mute button`)
      await toggleZoomMuteAsync()
      return true
    } else return false
  }
  const unmuteZoom = async () => {
    const zoomMute = await getZoomMute()
    if (zoomMute === 'muted') {
      console.log(`[unmuteZoom()]: unmuting Zoom by toggling the mute button`)
      await toggleZoomMuteAsync()
      return true
    } else return false
  }

  module.exports['getZoomMute'] = getZoomMute
  module.exports['muteZoom'] = muteZoom
  module.exports['unmuteZoom'] = unmuteZoom
} else {
  module.exports['getZoomMute'] = () => false
  module.exports['muteZoom'] = () => false
  module.exports['unmuteZoom'] = () => false
}
