// Martijn Smit <martijn@lostdomain.org / @smitmartijn>
#include "ZoomControls.h"

// #include <StreamDeckSDK/ESDLogger.h>
#include <windows.h>
#include <uiautomationclient.h>
#include <string>
#include <sstream>
#include <regex>

IUIAutomationElement *GetTopLevelWindowByName(LPWSTR windowName);
void rawListDescendants(IUIAutomationElement *pParent, std::string windowType);
void altPlusKey(UINT key);

std::string g_statusZoom = "stopped";
std::string g_statusMute = "unknown";
std::string g_statusVideo = "unknown";
std::string g_statusShare = "unknown";
std::string g_statusRecord = "unknown";
std::string g_windowName = "unknown";

IUIAutomation *g_pAutomation = nullptr;

void init()
{
  CoInitializeEx(NULL, COINIT_MULTITHREADED);
  auto hr = CoCreateInstance(__uuidof(CUIAutomation), NULL, CLSCTX_INPROC_SERVER, __uuidof(IUIAutomation), (void **)&g_pAutomation);
  if (FAILED(hr) || g_pAutomation == nullptr)
  {
    // ESDDebug("failed to init automation");
    return;
  }
}

std::string formatStatus()
{
  std::stringstream ss;
  ss << "zoomStatus:" << g_statusZoom << ",";
  ss << "zoomMute:" << g_statusMute << ",";
  ss << "zoomVideo:" << g_statusVideo << ",";
  ss << "zoomRecord:" << g_statusRecord << ",";
  ss << "zoomShare:" << g_statusShare;
  return ss.str();
}

std::string ZoomControls::osGetZoomStatus()
{
  init();

  // Get the handle of the Zoom window.
  HWND zoomWnd = ::FindWindow(NULL, "Zoom");
  if (zoomWnd != NULL)
  {
    g_statusZoom = "open";
    IUIAutomationElement *zoom = GetTopLevelWindowByName(L"Zoom");
    rawListDescendants(zoom, "Zoom");
    g_windowName = "Zoom";
  }

  // Get the handle of the Zoom Meetings window.
  zoomWnd = ::FindWindow(NULL, "Zoom Meeting");
  if (zoomWnd != NULL)
  {
    g_statusZoom = "call";
    IUIAutomationElement *zoom = GetTopLevelWindowByName(L"Zoom Meeting");
    rawListDescendants(zoom, "Zoom Meeting");
    g_windowName = "Zoom Meeting";
  }

  // Get the handle of the Meetings Controls window.
  zoomWnd = ::FindWindow(NULL, "Meeting Controls");
  if (zoomWnd != NULL)
  {
    g_statusZoom = "call-sharing";
    IUIAutomationElement *zoom = GetTopLevelWindowByName(L"Meeting Controls");
    rawListDescendants(zoom, "Meeting Controls");
    g_windowName = "Meeting Controls";
  }

  if (g_pAutomation != NULL)
    g_pAutomation->Release();

  CoUninitialize();

  return formatStatus();
}

Napi::String ZoomControls::osGetZoomStatusWrapped(const Napi::CallbackInfo &info)
{
  Napi::Env env = info.Env();
  Napi::String returnValue = Napi::String::New(env, ZoomControls::osGetZoomStatus());
  return returnValue;
}

IUIAutomationElement *GetTopLevelWindowByName(LPWSTR windowName)
{
  if (windowName == NULL)
  {
    return NULL;
  }

  VARIANT varProp;
  varProp.vt = VT_BSTR;
  varProp.bstrVal = SysAllocString(windowName);
  if (varProp.bstrVal == NULL)
  {
    return NULL;
  }

  IUIAutomationElement *pRoot = NULL;
  IUIAutomationElement *pFound = NULL;

  // Get the desktop element.
  HRESULT hr = g_pAutomation->GetRootElement(&pRoot);
  if (FAILED(hr) || pRoot == NULL)
  {
    goto cleanup;
  }

  // Get a top-level element by name, such as "Program Manager"
  IUIAutomationCondition *pCondition = NULL;
  hr = g_pAutomation->CreatePropertyCondition(UIA_NamePropertyId, varProp, &pCondition);
  if (FAILED(hr))
  {
    goto cleanup;
  }

  pRoot->FindFirst(TreeScope_Children, pCondition, &pFound);

cleanup:
  if (pRoot != NULL)
    pRoot->Release();

  if (pCondition != NULL)
    pCondition->Release();

  VariantClear(&varProp);

  return pFound;
}

void rawListDescendants(IUIAutomationElement *pParent, std::string windowType)
{
  if (pParent == NULL)
    return;

  IUIAutomationTreeWalker *pControlWalker = NULL;
  IUIAutomationElement *pNode = NULL;

  g_pAutomation->get_RawViewWalker(&pControlWalker);
  if (pControlWalker == NULL)
  {
    goto cleanup;
  }

  pControlWalker->GetFirstChildElement(pParent, &pNode);
  if (pNode == NULL)
  {
    goto cleanup;
  }

  while (pNode)
  {
    BSTR desc;
    BSTR sName;

    pNode->get_CurrentLocalizedControlType(&desc);
    pNode->get_CurrentName(&sName);

    // BSTR get_CurrentAcceleratorKey;
    // pNode->get_CurrentAcceleratorKey(&get_CurrentAcceleratorKey);
    // BSTR get_CurrentAccessKey;
    // pNode->get_CurrentAccessKey(&get_CurrentAccessKey);
    // BSTR get_CurrentAriaProperties;
    // pNode->get_CurrentAriaProperties(&get_CurrentAriaProperties);
    // BSTR get_CurrentAriaRole;
    // pNode->get_CurrentAriaRole(&get_CurrentAriaRole);
    // BSTR get_CurrentAutomationId;
    // pNode->get_CurrentAutomationId(&get_CurrentAutomationId);
    // RECT get_CurrentBoundingRectangle;
    // pNode->get_CurrentBoundingRectangle(&get_CurrentBoundingRectangle);
    // BSTR get_CurrentClassName;
    // pNode->get_CurrentClassName(&get_CurrentClassName);
    // CONTROLTYPEID get_CurrentControlType;
    // pNode->get_CurrentControlType(&get_CurrentControlType);
    // int get_CurrentCulture;
    // pNode->get_CurrentCulture(&get_CurrentCulture);
    // BSTR get_CurrentFrameworkId;
    // pNode->get_CurrentFrameworkId(&get_CurrentFrameworkId);
    // BOOL get_CurrentHasKeyboardFocus;
    // pNode->get_CurrentHasKeyboardFocus(&get_CurrentHasKeyboardFocus);
    // BSTR get_CurrentHelpText;
    // pNode->get_CurrentHelpText(&get_CurrentHelpText);
    // BOOL get_CurrentIsContentElement;
    // pNode->get_CurrentIsContentElement(&get_CurrentIsContentElement);
    // BOOL get_CurrentIsControlElement;
    // pNode->get_CurrentIsControlElement(&get_CurrentIsControlElement);
    // BOOL get_CurrentIsDataValidForForm;
    // pNode->get_CurrentIsDataValidForForm(&get_CurrentIsDataValidForForm);
    // BOOL get_CurrentIsEnabled;
    // pNode->get_CurrentIsEnabled(&get_CurrentIsEnabled);
    // BOOL get_CurrentIsKeyboardFocusable;
    // pNode->get_CurrentIsKeyboardFocusable(&get_CurrentIsKeyboardFocusable);
    // BOOL get_CurrentIsOffscreen;
    // pNode->get_CurrentIsOffscreen(&get_CurrentIsOffscreen);
    // BOOL get_CurrentIsPassword;
    // pNode->get_CurrentIsPassword(&get_CurrentIsPassword);
    // BOOL get_CurrentIsRequiredForForm;
    // pNode->get_CurrentIsRequiredForForm(&get_CurrentIsRequiredForForm);
    // BSTR get_CurrentItemStatus;
    // pNode->get_CurrentItemStatus(&get_CurrentItemStatus);
    // BSTR get_CurrentItemType;
    // pNode->get_CurrentItemType(&get_CurrentItemType);
    // UIA_HWND get_CurrentNativeWindowHandle;
    // pNode->get_CurrentNativeWindowHandle(&get_CurrentNativeWindowHandle);
    // OrientationType get_CurrentOrientation;
    // pNode->get_CurrentOrientation(&get_CurrentOrientation);
    // int get_CurrentProcessId;
    // pNode->get_CurrentProcessId(&get_CurrentProcessId);
    // BSTR get_CurrentProviderDescription;
    // pNode->get_CurrentProviderDescription(&get_CurrentProviderDescription);

    /*if (DEBUG)
    {
      std::cout << "Element type = ";
      if (NULL == desc)
        std::cout << "Empty";
      else
        std::wcout << desc;

      std::cout << "  Name  = ";
      if (NULL == sName)
        std::cout << "Empty";
      else
        std::wcout << sName;

      std::cout << "\n";
    }*/

    // we have a name of the element - match it against what we're looking for
    if (sName != NULL)
    {
      std::wstring strName(sName, SysStringLen(sName));

      // we're looking for different buttons per window. This is the Main Zoom window
      if (windowType == "Zoom Meeting")
      {
        if (strName.find(L"Mute") != std::string::npos)
        {
          g_statusMute = "unmuted";
        }
        if (strName.find(L"Unmute") != std::string::npos)
        {
          g_statusMute = "muted";
        }

        if (strName.find(L"start my video") != std::string::npos)
        {
          g_statusVideo = "stopped";
        }
        if (strName.find(L"stop my video") != std::string::npos)
        {
          g_statusVideo = "started";
        }

        if (strName.find(L"Share Screen") != std::string::npos)
        {
          g_statusShare = "stopped";
        }

        if (strName.find(L"Record") != std::string::npos && g_statusRecord != "started")
        {
          g_statusRecord = "stopped";
        }
        if (strName.find(L"Stop Recording") != std::string::npos)
        {
          g_statusRecord = "started";
        }
      }
      // we're looking for different buttons per window. This is the minimized Zoom window
      else if (windowType == "Zoom")
      {
        if (strName.find(L"Mute My Audio") != std::string::npos)
        {
          g_statusMute = "unmuted";
        }
        if (strName.find(L"Unmute My Audio") != std::string::npos)
        {
          g_statusMute = "muted";
        }

        if (strName.find(L"Start Video") != std::string::npos)
        {
          g_statusVideo = "stopped";
        }
        if (strName.find(L"Stop Video") != std::string::npos)
        {
          g_statusVideo = "started";
        }

        // you cannot minimize the window while recording
        g_statusRecord = "stopped";
        g_statusShare = "stopped";
      }

      // we're looking for different buttons per window. This is the Zoom window when you're sharing
      else if (windowType == "Meeting Controls")
      {
        if (strName.find(L"currently unmuted") != std::string::npos)
        {
          g_statusMute = "unmuted";
        }
        if (strName.find(L"currently muted") != std::string::npos)
        {
          g_statusMute = "muted";
        }

        if (strName.find(L"Start Video") != std::string::npos)
        {
          g_statusVideo = "stopped";
        }
        if (strName.find(L"Stop Video") != std::string::npos)
        {
          g_statusVideo = "started";
        }

        // toolbar open = sharing
        g_statusShare = "started";
        // can't read record button in this view
        g_statusRecord = "disabled";
      }
    }

    // only go into windows and panes
    if (desc != NULL)
      if (0 == wcscmp(desc, L"window") || 0 == wcscmp(desc, L"pane"))
        rawListDescendants(pNode, windowType);

    if (desc != NULL)
      SysFreeString(desc);
    if (sName != NULL)
      SysFreeString(sName);

    // get the next element
    IUIAutomationElement *pNext;
    pControlWalker->GetNextSiblingElement(pNode, &pNext);
    pNode->Release();
    pNode = pNext;
  }

cleanup:
  if (pControlWalker != NULL)
    pControlWalker->Release();

  if (pNode != NULL)
    pNode->Release();

  return;
}

std::string ZoomControls::osFocusZoomWindow()
{
  // Get the handle of the Zoom window.
  HWND zoomWnd = ::FindWindow(NULL, g_windowName.c_str());
  if (zoomWnd == NULL)
  {
    return NULL;
  }
  SetForegroundWindow(zoomWnd);
  return ZoomControls::osGetZoomStatus();
}

Napi::String ZoomControls::osFocusZoomWindowWrapped(const Napi::CallbackInfo &info)
{
  Napi::Env env = info.Env();
  Napi::String returnValue = Napi::String::New(env, ZoomControls::osFocusZoomWindow());
  return returnValue;
}

std::string ZoomControls::osToggleZoomMute()
{
  //ESDDebug("ZoomPlugin: Sending mute shortcut");
  ZoomControls::osFocusZoomWindow();
  ::keybd_event(VK_MENU, 0, 0, 0);
  ::keybd_event(0x41, 0, 0, 0); // virtual key code for 'A'
  keybd_event(VK_MENU, 0, KEYEVENTF_KEYUP, 0);
  keybd_event(0x41, 0, KEYEVENTF_KEYUP, 0); // virtual key code for 'A'
  return ZoomControls::osGetZoomStatus();
}

Napi::String ZoomControls::osToggleZoomMuteWrapped(const Napi::CallbackInfo &info)
{
  Napi::Env env = info.Env();
  Napi::String returnValue = Napi::String::New(env, ZoomControls::osToggleZoomMute());
  return returnValue;
}

std::string ZoomControls::osToggleZoomShare()
{
  //ESDDebug("ZoomPlugin: Sending share shortcut");
  ZoomControls::osFocusZoomWindow();
  ::keybd_event(VK_MENU, 0, 0, 0);
  ::keybd_event(0x53, 0, 0, 0); // virtual key code for 'S'
  keybd_event(VK_MENU, 0, KEYEVENTF_KEYUP, 0);
  keybd_event(0x53, 0, KEYEVENTF_KEYUP, 0); // virtual key code for 'S'
  return ZoomControls::osGetZoomStatus();
}

Napi::String ZoomControls::osToggleZoomShareWrapped(const Napi::CallbackInfo &info)
{
  Napi::Env env = info.Env();
  Napi::String returnValue = Napi::String::New(env, ZoomControls::osToggleZoomShare());
  return returnValue;
}

std::string ZoomControls::osToggleZoomVideo()
{
  //ESDDebug("ZoomPlugin: Sending video shortcut");
  ZoomControls::osFocusZoomWindow();
  ::keybd_event(VK_MENU, 0, 0, 0);
  ::keybd_event(0x56, 0, 0, 0); // virtual key code for 'V'
  keybd_event(VK_MENU, 0, KEYEVENTF_KEYUP, 0);
  keybd_event(0x56, 0, KEYEVENTF_KEYUP, 0); // virtual key code for 'V'
  return ZoomControls::osGetZoomStatus();
}

Napi::String ZoomControls::osToggleZoomVideoWrapped(const Napi::CallbackInfo &info)
{
  Napi::Env env = info.Env();
  Napi::String returnValue = Napi::String::New(env, ZoomControls::osToggleZoomVideo());
  return returnValue;
}

std::string ZoomControls::osLeaveZoomMeeting()
{
  //ESDDebug("ZoomPlugin: Ending meeting");
  ZoomControls::osFocusZoomWindow();
  ::keybd_event(VK_MENU, 0, 0, 0);
  ::keybd_event(0x51, 0, 0, 0); // virtual key code for 'Q'
  keybd_event(VK_MENU, 0, KEYEVENTF_KEYUP, 0);
  keybd_event(0x51, 0, KEYEVENTF_KEYUP, 0); // virtual key code for 'Q'
  return ZoomControls::osGetZoomStatus();
}

Napi::String ZoomControls::osLeaveZoomMeetingWrapped(const Napi::CallbackInfo &info)
{
  Napi::Env env = info.Env();
  Napi::String returnValue = Napi::String::New(env, ZoomControls::osLeaveZoomMeeting());
  return returnValue;
}

std::string ZoomControls::osToggleZoomRecordCloud()
{
  //ESDDebug("ZoomPlugin: Toggling Cloud Recording");
  ZoomControls::osFocusZoomWindow();
  ::keybd_event(VK_MENU, 0, 0, 0);
  ::keybd_event(0x43, 0, 0, 0); // virtual key code for 'C'
  keybd_event(VK_MENU, 0, KEYEVENTF_KEYUP, 0);
  keybd_event(0x43, 0, KEYEVENTF_KEYUP, 0); // virtual key code for 'C'
  return ZoomControls::osGetZoomStatus();
}

Napi::String ZoomControls::osToggleZoomRecordCloudWrapped(const Napi::CallbackInfo &info)
{
  Napi::Env env = info.Env();
  Napi::String returnValue = Napi::String::New(env, ZoomControls::osToggleZoomRecordCloud());
  return returnValue;
}

std::string ZoomControls::osToggleZoomRecordLocal()
{
  //ESDDebug("ZoomPlugin: Toggling Local Recording");
  ZoomControls::osFocusZoomWindow();

  ::keybd_event(VK_MENU, 0, 0, 0);
  ::keybd_event(0x52, 0, 0, 0); // virtual key code for 'R'
  keybd_event(VK_MENU, 0, KEYEVENTF_KEYUP, 0);
  keybd_event(0x52, 0, KEYEVENTF_KEYUP, 0); // virtual key code for 'R'
  return ZoomControls::osGetZoomStatus();
}

Napi::String ZoomControls::osToggleZoomRecordLocalWrapped(const Napi::CallbackInfo &info)
{
  Napi::Env env = info.Env();
  Napi::String returnValue = Napi::String::New(env, ZoomControls::osToggleZoomRecordLocal());
  return returnValue;
}

std::string ZoomControls::osMuteAll()
{
  ZoomControls::osFocusZoomWindow();

  ::keybd_event(VK_MENU, 0, 0, 0);
  ::keybd_event(0x4D, 0, 0, 0); // virtual key code for 'M'
  keybd_event(VK_MENU, 0, KEYEVENTF_KEYUP, 0);
  keybd_event(0x4D, 0, KEYEVENTF_KEYUP, 0); // virtual key code for 'M'

  Sleep(200);
  // wait 200ms and press enter to negate the "are you sure?" window
  int key_count = 2;

  INPUT *input = new INPUT[key_count];
  for (int i = 0; i < key_count; i++)
  {
    input[i].ki.dwFlags = 0;
    input[i].type = INPUT_KEYBOARD;
    input[0].ki.wVk = VK_RETURN;
    input[0].ki.wScan = MapVirtualKey(VK_RETURN, MAPVK_VK_TO_VSC);
  }

  // key up
  input[1].ki.dwFlags = KEYEVENTF_KEYUP;

  SendInput(key_count, (LPINPUT)input, sizeof(INPUT));
  return ZoomControls::osGetZoomStatus();
}

Napi::String ZoomControls::osMuteAllWrapped(const Napi::CallbackInfo &info)
{
  Napi::Env env = info.Env();
  Napi::String returnValue = Napi::String::New(env, ZoomControls::osMuteAll());
  return returnValue;
}

std::string ZoomControls::osUnmuteAll()
{
  ZoomControls::osFocusZoomWindow();

  ::keybd_event(VK_MENU, 0, 0, 0);
  ::keybd_event(0x4D, 0, 0, 0); // virtual key code for 'M'
  keybd_event(VK_MENU, 0, KEYEVENTF_KEYUP, 0);
  keybd_event(0x4D, 0, KEYEVENTF_KEYUP, 0); // virtual key code for 'M'
  return ZoomControls::osGetZoomStatus();
}

Napi::String ZoomControls::osUnmuteAllWrapped(const Napi::CallbackInfo &info)
{
  Napi::Env env = info.Env();
  Napi::String returnValue = Napi::String::New(env, ZoomControls::osUnmuteAll());
  return returnValue;
}

Napi::Object ZoomControls::Init(Napi::Env env, Napi::Object exports)
{
  exports.Set("osGetZoomStatus", Napi::Function::New(env, ZoomControls::osGetZoomStatusWrapped));
  exports.Set("osFocusZoomWindow", Napi::Function::New(env, ZoomControls::osFocusZoomWindowWrapped));
  exports.Set("osToggleZoomMute", Napi::Function::New(env, ZoomControls::osToggleZoomMuteWrapped));
  exports.Set("osToggleZoomShare", Napi::Function::New(env, ZoomControls::osToggleZoomShareWrapped));
  exports.Set("osToggleZoomVideo", Napi::Function::New(env, ZoomControls::osToggleZoomVideoWrapped));
  exports.Set("osLeaveZoomMeeting", Napi::Function::New(env, ZoomControls::osLeaveZoomMeetingWrapped));
  exports.Set("osToggleZoomRecordCloud", Napi::Function::New(env, ZoomControls::osToggleZoomRecordCloudWrapped));
  exports.Set("osToggleZoomRecordLocal", Napi::Function::New(env, ZoomControls::osToggleZoomRecordLocalWrapped));
  exports.Set("osMuteAll", Napi::Function::New(env, ZoomControls::osMuteAllWrapped));
  exports.Set("osUnmuteAll", Napi::Function::New(env, ZoomControls::osUnmuteAllWrapped));
  return exports;
}
