const { ipcRenderer, app, remote } = require('electron')

function byOS(handlers, fallback) {
  const handler = handlers[process.platform]
  console.log(handler)
  if (!handler) {
    if (typeof fallback === 'function') return fallback()
    return fallback
  }
  if (typeof handler === 'function') return handler()
  return handler
}

function getOS() {
  return process.platform
}

window.byOS = byOS
window.getOS = getOS

let winAudio = null
let macAudio = null
let linuxAudio = null

if (getOS() === 'win32') {
  try {
    winAudio = require('win-audio').mic
  } catch (error) {
    winAudio = null
  }
}

if (getOS() === 'darwin') {
  try {
    macAudio = require('@muteme/mac-audio')
  } catch (error) {
    macAudio = null
  }
}

if (getOS() === 'linux') {
  try {
    linuxAudio = require('@muteme/linux-audio')
  } catch (error) {
    linuxAudio = null
  }
}

const macIsMuted = async () => {
  const { id: deviceId } = await macAudio.getDefaultInputDevice()
  const isMuted = await macAudio.getDeviceMute(deviceId)
  return isMuted
}

const macMute = async () => {
  const { id: deviceId } = await macAudio.getDefaultInputDevice()
  const isMuted = await macAudio.getDeviceMute(deviceId)
  if (!isMuted) {
    await macAudio.toggleDeviceMute(deviceId)
    return true
  }
  return false
}
const macUnmute = async () => {
  const { id: deviceId } = await macAudio.getDefaultInputDevice()
  const isMuted = await macAudio.getDeviceMute(deviceId)
  if (isMuted) {
    await macAudio.toggleDeviceMute(deviceId)
    return true
  }
  return false
}

const microphone = {
  isMuted: winAudio
    ? winAudio.isMuted
    : macAudio
    ? macIsMuted
    : linuxAudio
    ? linuxAudio.isMuted
    : () => null,
  mute: winAudio ? winAudio.mute : macAudio ? macMute : linuxAudio ? linuxAudio.mute : () => null,
  unmute: winAudio
    ? winAudio.unmute
    : macAudio
    ? macUnmute
    : linuxAudio
    ? linuxAudio.unmute
    : () => null
}

const zoomControl = require('@muteme/zoom-control')
const customTitlebar = require('custom-electron-titlebar')
const HID = require('node-hid')

const { Menu } = remote

window.ipcRenderer = ipcRenderer
window.app = app
window.microphone = microphone
window.HID = HID
window.zoomControl = zoomControl
window.clientVersion = remote.app.getVersion()

// if (['win32'].includes(process.platform)) {
//   window.addEventListener('DOMContentLoaded', () => {
//     const menu = new Menu()
//     const MyTitleBar = new customTitlebar.Titlebar({
//       backgroundColor: customTitlebar.Color.fromHex('#282c34'),
//       menu,
//       titleHorizontalAlignment: 'left'
//     })

//     MyTitleBar.updateTitle('MuteMe Client')
//   })
// }
