#!/bin/sh

# Define the path to the udev rules directory
RULES_PATH="/etc/udev/rules.d"
RULES_FILE="$RULES_PATH/muteme.rules"

echo "Working with udev rules file at: $RULES_FILE"

cd $RULES_PATH
EXPECTED_RULES_CONTENT=""
CURRENT_RULES_CONTENT=$(cat $RULES_FILE 2>/dev/null)  # Suppress errors if file does not exist

echo "Current Rules Content:"
echo "$CURRENT_RULES_CONTENT"

# Build expected rules content from provided arguments
while [ $# -gt 0 ]; do
    VENDOR_ID=$1
    shift
    PRODUCT_ID=$1
    shift
    RULE="SUBSYSTEM==\"usb\", ATTRS{idVendor}==\"$VENDOR_ID\", ATTRS{idProduct}==\"$PRODUCT_ID\", MODE=\"0666\", GROUP=\"plugdev\"
KERNEL==\"hidraw*\", ATTRS{idVendor}==\"$VENDOR_ID\", ATTRS{idProduct}==\"$PRODUCT_ID\", MODE=\"0666\", GROUP=\"plugdev\""
    EXPECTED_RULES_CONTENT="$EXPECTED_RULES_CONTENT
$RULE"
done

echo "Expected Rules Content:"
echo "$EXPECTED_RULES_CONTENT"

# Normalize and compare content
if [ "$(echo "$EXPECTED_RULES_CONTENT" | sort | tr -d '[:space:]')" = "$(echo "$CURRENT_RULES_CONTENT" | sort | tr -d '[:space:]')" ]; then
    echo "No changes required"
else
    echo "Changes required"
fi
